@@ -18,9 +18,8 @@ from localstack.services.awslambda import lambda_api
 from localstack.services.apigateway import helpers
 from localstack.services.generic_proxy import ProxyListener
 from localstack.utils.aws.aws_responses import flask_to_requests_response, requests_response, LambdaResponse
-from localstack.services.apigateway.helpers import (get_resource_for_path,
-    handle_authorizers, extract_query_string_params,
-    extract_path_params, make_error_response, get_cors_response)
+from localstack.services.apigateway.helpers import (get_resource_for_path, handle_authorizers,
+    extract_query_string_params, extract_path_params, make_error_response, get_cors_response)
 
 # set up logger
 LOGGER = logging.getLogger(__name__)
@@ -41,14 +40,7 @@ class AuthorizationError(Exception):
 class ProxyListenerApiGateway(ProxyListener):
     def forward_request(self, method, path, data, headers):
         if re.match(PATH_REGEX_USER_REQUEST, path):
-            search_match = re.search(PATH_REGEX_USER_REQUEST, path)
-            api_id = search_match.group(1)
-            stage = search_match.group(2)
-            relative_path_w_query_params = '/%s' % search_match.group(3)
-            try:
-                return invoke_rest_api(api_id, stage, method, relative_path_w_query_params, data, headers, path=path)
-            except AuthorizationError as e:
-                return make_error_response('Not authorized to invoke REST API %s: %s' % (api_id, e), 403)
+            return invoke_rest_api_from_request(method, path, data, headers)
 
         data = data and json.loads(to_str(data))
 
@@ -218,7 +210,19 @@ def apply_template(integration, req_res_type, data, path_params={}, query_params
     return data
 
 
-def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=None):
+def invoke_rest_api_from_request(method, path, data, headers, context={}):
+    search_match = re.search(PATH_REGEX_USER_REQUEST, path)
+    api_id = search_match.group(1)
+    stage = search_match.group(2)
+    relative_path_w_query_params = '/%s' % search_match.group(3)
+    try:
+        return invoke_rest_api(api_id, stage, method, relative_path_w_query_params,
+            data, headers, path=path, context=context)
+    except AuthorizationError as e:
+        return make_error_response('Not authorized to invoke REST API %s: %s' % (api_id, e), 403)
+
+
+def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=None, context={}):
     path = path or invocation_path
     relative_path, query_string_params = extract_query_string_params(path=invocation_path)
 
@@ -246,15 +250,14 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
         return make_error_response('Unable to find integration for path %s' % path, 404)
 
     uri = integration.get('uri') or ''
+    integration_type = integration['type'].upper()
 
     if uri.startswith('arn:aws:apigateway:') and ':lambda:path' in uri:
-        if integration['type'] in ['AWS', 'AWS_PROXY']:
+        if integration_type in ['AWS', 'AWS_PROXY']:
             func_arn = uri.split(':lambda:path')[1].split('functions/')[1].split('/invocations')[0]
             data_str = json.dumps(data) if isinstance(data, (dict, list)) else to_str(data)
             account_id = uri.split(':lambda:path')[1].split(':function:')[0].split(':')[-1]
-            source_ip = headers['X-Forwarded-For'].split(',')[-2]
-            integration_method = integration.get('httpMethod')
-            integration_method = method if integration_method in [None, 'ANY'] else integration_method
+            source_ip = headers['X-Forwarded-For'].split(',')[-2].strip()
 
             try:
                 path_params = extract_path_params(path=relative_path, extracted_path=extracted_path)
@@ -279,17 +282,15 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
                     'sourceIp': source_ip,
                     'userAgent': headers['User-Agent'],
                 },
-                'httpMethod': integration_method,
+                'httpMethod': method,
                 'protocol': 'HTTP/1.1',
                 'requestTime': datetime.datetime.utcnow(),
                 'requestTimeEpoch': int(time.time() * 1000),
             }
 
-            result = lambda_api.process_apigateway_invocation(func_arn, relative_path, data_str, stage, api_id,
-                                                              headers, path_params=path_params,
-                                                              query_string_params=query_string_params,
-                                                              method=integration_method, resource_path=path,
-                                                              request_context=request_context)
+            result = lambda_api.process_apigateway_invocation(func_arn, relative_path, data_str,
+                stage, api_id, headers, path_params=path_params, query_string_params=query_string_params,
+                method=method, resource_path=path, request_context=request_context, event_context=context)
 
             if isinstance(result, FlaskResponse):
                 response = flask_to_requests_response(result)
@@ -297,7 +298,7 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
                 response = result
             else:
                 response = LambdaResponse()
-                parsed_result = result if isinstance(result, dict) else json.loads(str(result))
+                parsed_result = result if isinstance(result, dict) else json.loads(str(result or '{}'))
                 parsed_result = common.json_safe(parsed_result)
                 parsed_result = {} if parsed_result is None else parsed_result
                 response.status_code = int(parsed_result.get('statusCode', 200))
@@ -324,7 +325,7 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
         LOGGER.warning(msg)
         return make_error_response(msg, 404)
 
-    elif integration['type'] == 'AWS':
+    elif integration_type == 'AWS':
         if 'kinesis:action/' in uri:
             if uri.endswith('kinesis:action/PutRecords'):
                 target = kinesis_listener.ACTION_PUT_RECORDS
@@ -358,7 +359,7 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
         LOGGER.warning(msg)
         return make_error_response(msg, 404)
 
-    elif integration['type'] == 'AWS_PROXY':
+    elif integration_type == 'AWS_PROXY':
         if uri.startswith('arn:aws:apigateway:') and ':dynamodb:action' in uri:
             # arn:aws:apigateway:us-east-1:dynamodb:action/PutItem&Table=MusicCollection
             table_name = uri.split(':dynamodb:action')[1].split('&Table=')[1]
@@ -395,7 +396,7 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
             LOGGER.warning(msg)
             return make_error_response(msg, 404)
 
-    elif integration['type'] in ['HTTP_PROXY', 'HTTP']:
+    elif integration_type in ['HTTP_PROXY', 'HTTP']:
         function = getattr(requests, method.lower())
 
         # apply custom request template
@@ -411,6 +412,14 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
 
         return result
 
+    elif integration_type == 'MOCK':
+        # TODO: add logic for MOCK responses
+        pass
+
+    if method == 'OPTIONS':
+        # fall back to returning CORS headers if this is an OPTIONS request
+        return get_cors_response(headers)
+
     msg = ('API Gateway integration type "%s", method "%s", URI "%s" not yet implemented' %
            (integration['type'], method, integration.get('uri')))
     LOGGER.warning(msg)
