@@ -23,7 +23,7 @@ from docopt import docopt
 from localstack import config, constants
 from localstack.utils import bootstrap
 from localstack.utils.bootstrap import (
-    start_infra_in_docker, start_infra_locally, run, MAIN_CONTAINER_NAME, docker_container_running)
+    start_infra_in_docker, start_infra_locally, run, docker_container_running)
 
 # Note: make sure we don't have other imports at the root level here
 
@@ -93,10 +93,10 @@ Commands:
 Options:
     """
     args.update(docopt(cmd_ssh.__doc__.strip(), argv=argv))
-    if not docker_container_running(MAIN_CONTAINER_NAME):
-        raise Exception('Expected 1 running "%s" container, but found none' % MAIN_CONTAINER_NAME)
+    if not docker_container_running(config.MAIN_CONTAINER_NAME):
+        raise Exception('Expected 1 running "%s" container, but found none' % config.MAIN_CONTAINER_NAME)
     try:
-        process = run('docker exec -it %s bash' % MAIN_CONTAINER_NAME, tty=True)
+        process = run('docker exec -it %s bash' % config.MAIN_CONTAINER_NAME, tty=True)
         process.wait()
     except KeyboardInterrupt:
         pass
