@@ -6,6 +6,7 @@ import logging
 import traceback
 import moto.cloudformation.utils
 from six import iteritems
+from urllib.parse import urlparse
 from localstack.utils import common
 from localstack.utils.aws import aws_stack
 from localstack.constants import AWS_REGION_US_EAST_1
@@ -16,6 +17,7 @@ from localstack.services.awslambda.lambda_api import get_handler_file_from_name
 ACTION_CREATE = 'create'
 ACTION_DELETE = 'delete'
 PLACEHOLDER_RESOURCE_NAME = '__resource_name__'
+PLACEHOLDER_AWS_NO_VALUE = '__aws_no_value__'
 
 LOG = logging.getLogger(__name__)
 
@@ -220,6 +222,52 @@ def param_defaults(param_func, defaults):
     return replace
 
 
+def iam_create_policy_params(params, **kwargs):
+    result = {'PolicyName': params['PolicyName']}
+    policy_doc = remove_none_values(params['PolicyDocument'])
+    result['PolicyDocument'] = json.dumps(policy_doc)
+    return result
+
+
+def get_ddb_provisioned_throughput(params, **kwargs):
+    args = params.get('ProvisionedThroughput')
+    if args:
+        if isinstance(args['ReadCapacityUnits'], str):
+            args['ReadCapacityUnits'] = int(args['ReadCapacityUnits'])
+        if isinstance(args['WriteCapacityUnits'], str):
+            args['WriteCapacityUnits'] = int(args['WriteCapacityUnits'])
+    return args
+
+
+def get_ddb_global_sec_indexes(params, **kwargs):
+    args = params.get('GlobalSecondaryIndexes')
+    if args:
+        for index in args:
+            provisoned_throughput = index['ProvisionedThroughput']
+            if isinstance(provisoned_throughput['ReadCapacityUnits'], str):
+                provisoned_throughput['ReadCapacityUnits'] = int(provisoned_throughput['ReadCapacityUnits'])
+            if isinstance(provisoned_throughput['WriteCapacityUnits'], str):
+                provisoned_throughput['WriteCapacityUnits'] = int(provisoned_throughput['WriteCapacityUnits'])
+    return args
+
+
+def get_apigw_resource_params(params, **kwargs):
+    result = {
+        'restApiId': params.get('RestApiId'),
+        'pathPart': params.get('PathPart'),
+        'parentId': params.get('ParentId')
+    }
+    if not result.get('parentId'):
+        # get root resource id
+        apigw = aws_stack.connect_to_service('apigateway')
+        resources = apigw.get_resources(restApiId=result['restApiId'])['items']
+        root_resource = ([r for r in resources if r['path'] == '/'] or [None])[0]
+        if not root_resource:
+            raise Exception('Unable to find root resource for REST API %s' % result['restApiId'])
+        result['parentId'] = root_resource['id']
+    return result
+
+
 # maps resource types to functions and parameters for creation
 RESOURCE_TO_FUNCTION = {
     'S3::Bucket': {
@@ -406,9 +454,9 @@ RESOURCE_TO_FUNCTION = {
                 'TableName': 'TableName',
                 'AttributeDefinitions': 'AttributeDefinitions',
                 'KeySchema': 'KeySchema',
-                'ProvisionedThroughput': 'ProvisionedThroughput',
+                'ProvisionedThroughput': get_ddb_provisioned_throughput,
                 'LocalSecondaryIndexes': 'LocalSecondaryIndexes',
-                'GlobalSecondaryIndexes': 'GlobalSecondaryIndexes',
+                'GlobalSecondaryIndexes': get_ddb_global_sec_indexes,
                 'StreamSpecification': lambda params, **kwargs: (
                     common.merge_dicts(params.get('StreamSpecification'), {'StreamEnabled': True}, default=None))
             },
@@ -464,6 +512,13 @@ RESOURCE_TO_FUNCTION = {
             }
         }
     },
+    'IAM::Policy': {
+        'create': {
+            'function': 'create_policy',
+            # TODO: associate policies with users, groups, roles
+            'parameters': iam_create_policy_params
+        }
+    },
     'ApiGateway::RestApi': {
         'create': {
             'function': 'create_rest_api',
@@ -471,16 +526,18 @@ RESOURCE_TO_FUNCTION = {
                 'name': 'Name',
                 'description': 'Description'
             }
+        },
+        'delete': {
+            'function': 'delete_rest_api',
+            'parameters': {
+                'restApiId': 'PhysicalResourceId',
+            }
         }
     },
     'ApiGateway::Resource': {
         'create': {
             'function': 'create_resource',
-            'parameters': {
-                'restApiId': 'RestApiId',
-                'pathPart': 'PathPart',
-                'parentId': 'ParentId'
-            }
+            'parameters': get_apigw_resource_params
         }
     },
     'ApiGateway::Method': {
@@ -497,6 +554,8 @@ RESOURCE_TO_FUNCTION = {
     },
     'ApiGateway::Method::Integration': {
     },
+    'ApiGateway::Account': {
+    },
     'ApiGateway::Deployment': {
         'create': {
             'function': 'create_deployment',
@@ -582,6 +641,7 @@ RESOURCE_TO_FUNCTION = {
     }
 }
 
+
 # ----------------
 # UTILITY METHODS
 # ----------------
@@ -705,9 +765,8 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
     resource_props = resource.get('Properties')
     try:
         if resource_type == 'Lambda::Function':
-            resource_props['FunctionName'] = (resource_props.get('FunctionName') or
-                '{}-lambda-{}'.format(stack_name[:45], common.short_uid()))
-            resource_id = resource_props['FunctionName'] if resource else resource_id
+            func_name = resolve_refs_recursively(stack_name, resource_props['FunctionName'], resources)
+            resource_id = func_name if resource else resource_id
             return aws_stack.connect_to_service('lambda').get_function(FunctionName=resource_id)
         elif resource_type == 'Lambda::Version':
             name = resource_props.get('FunctionName')
@@ -753,7 +812,7 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
         elif resource_type == 'ApiGateway::Resource':
             api_id = resource_props['RestApiId'] if resource else resource_id
             api_id = resolve_refs_recursively(stack_name, api_id, resources)
-            parent_id = resolve_refs_recursively(stack_name, resource_props['ParentId'], resources)
+            parent_id = resolve_refs_recursively(stack_name, resource_props.get('ParentId'), resources)
             if not api_id or not parent_id:
                 return None
             api_resources = aws_stack.connect_to_service('apigateway').get_resources(restApiId=api_id)['items']
@@ -924,6 +983,8 @@ def resolve_ref(stack_name, ref, resources, attribute):
         return 'aws'
     if ref == 'AWS::StackName':
         return stack_name
+    if ref == 'AWS::NoValue':
+        return PLACEHOLDER_AWS_NO_VALUE
 
     # first, check stack parameters
     stack_param = get_stack_parameter(stack_name, ref)
@@ -1082,6 +1143,9 @@ def remove_none_values(params):
             for k, v in dict(o).items():
                 if v is None:
                     o.pop(k)
+        if isinstance(o, list):
+            common.run_safe(o.remove, None)
+            common.run_safe(o.remove, PLACEHOLDER_AWS_NO_VALUE)
         return o
     result = common.recurse_object(params, remove_nones)
     return result
@@ -1146,6 +1210,15 @@ def configure_resource_via_sdk(resource_id, resources, resource_type, func_detai
         resource['Properties'] = {}
     resource_props = resource['Properties']
 
+    # Validate props for each resource type
+    if resource_type == 'Lambda::Function':
+        # Properties will be validated by botocore before sending request to AWS
+        # botocore/data/lambda/2015-03-31/service-2.json:1161 (EnvironmentVariableValue)
+        # https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-environment.html
+        if 'Environment' in resource_props:
+            environment_variables = resource_props['Environment'].get('Variables', {})
+            resource_props['Environment']['Variables'] = {k: str(v) for k, v in environment_variables.items()}
+
     if callable(params):
         params = params(resource_props, stack_name=stack_name, resources=resources)
     else:
@@ -1220,11 +1293,27 @@ def configure_resource_via_sdk(resource_id, resources, resource_type, func_detai
             kwargs = {}
             if integration.get('Uri'):
                 uri = resolve_refs_recursively(stack_name, integration.get('Uri'), resources)
+
+                # Moto has a validate method on Uri for integration_type "HTTP" | "HTTP_PROXY" that does not accept
+                # Uri value without path, we need to add path ("/") if not exists
+                if integration.get('Type') in ['HTTP', 'HTTP_PROXY']:
+                    rs = urlparse(uri)
+                    if not rs.path:
+                        uri = '{}/'.format(uri)
+
                 kwargs['uri'] = uri
+
             if integration.get('IntegrationHttpMethod'):
                 kwargs['integrationHttpMethod'] = integration['IntegrationHttpMethod']
-            apigateway.put_integration(restApiId=api_id, resourceId=res_id,
-                httpMethod=resource_props['HttpMethod'], type=integration['Type'], **kwargs)
+
+            apigateway.put_integration(
+                restApiId=api_id,
+                resourceId=res_id,
+                httpMethod=resource_props['HttpMethod'],
+                type=integration['Type'],
+                **kwargs
+            )
+
         responses = resource_props.get('MethodResponses') or []
         for response in responses:
             api_id = resolve_refs_recursively(stack_name, resource_props['RestApiId'], resources)
@@ -1233,6 +1322,12 @@ def configure_resource_via_sdk(resource_id, resources, resource_type, func_detai
                 httpMethod=resource_props['HttpMethod'], statusCode=str(response['StatusCode']),
                 responseParameters=response.get('ResponseParameters', {}))
 
+    elif resource_type == 'ApiGateway::RestApi':
+        body = resource_props.get('Body')
+        if body:
+            body = json.dumps(body) if isinstance(body, dict) else body
+            client.put_rest_api(restApiId=result['id'], body=common.to_bytes(body))
+
     elif resource_type == 'SNS::Topic':
         subscriptions = resource_props.get('Subscription', [])
         for subscription in subscriptions:
