@@ -48,7 +48,7 @@ CACHE_CLEAN_TIMEOUT = 60 * 5
 CACHE_MAX_AGE = 60 * 60
 CACHE_FILE_PATTERN = os.path.join(tempfile.gettempdir(), '_random_dir_', 'cache.*.json')
 last_cache_clean_time = {'time': 0}
-mutex_clean = threading.Semaphore(1)
+MUTEX_CLEAN = threading.Semaphore(1)
 
 # misc. constants
 TIMESTAMP_FORMAT = '%Y-%m-%dT%H:%M:%S'
@@ -462,6 +462,15 @@ def get_service_protocol():
     return 'https' if config.USE_SSL else 'http'
 
 
+def edge_ports_info():
+    if config.EDGE_PORT_HTTP:
+        result = 'ports %s/%s' % (config.EDGE_PORT, config.EDGE_PORT_HTTP)
+    else:
+        result = 'port %s' % config.EDGE_PORT
+    result = '%s %s' % (get_service_protocol(), result)
+    return result
+
+
 def timestamp(time=None, format=TIMESTAMP_FORMAT):
     if not time:
         time = datetime.utcnow()
@@ -709,12 +718,13 @@ def is_linux():
 
 def is_alpine():
     try:
-        if '_is_alpine_' not in CACHE:
-            CACHE['_is_alpine_'] = False
-            if not os.path.exists('/etc/issue'):
-                return False
-            out = to_str(subprocess.check_output('cat /etc/issue', shell=True))
-            CACHE['_is_alpine_'] = 'Alpine' in out
+        with MUTEX_CLEAN:
+            if '_is_alpine_' not in CACHE:
+                CACHE['_is_alpine_'] = False
+                if not os.path.exists('/etc/issue'):
+                    return False
+                out = to_str(subprocess.check_output('cat /etc/issue', shell=True))
+                CACHE['_is_alpine_'] = 'Alpine' in out
     except subprocess.CalledProcessError:
         return False
     return CACHE['_is_alpine_']
@@ -1049,7 +1059,7 @@ def generate_ssl_cert(target_file=None, overwrite=False, random=False, return_co
     cert.gmtime_adj_notAfter(2 * 365 * 24 * 60 * 60)
     cert.set_issuer(cert.get_subject())
     cert.set_pubkey(k)
-    alt_names = b'DNS:localhost,DNS:test.localhost.atlassian.io,IP:127.0.0.1'
+    alt_names = b'DNS:localhost,DNS:test.localhost.atlassian.io,DNS:localhost.localstack.cloud,IP:127.0.0.1'
     cert.add_extensions([
         crypto.X509Extension(b'subjectAltName', False, alt_names),
         crypto.X509Extension(b'basicConstraints', True, b'CA:false'),
@@ -1093,9 +1103,10 @@ def generate_ssl_cert(target_file=None, overwrite=False, random=False, return_co
     return file_content
 
 
-def run_safe(_python_lambda, print_error=False, **kwargs):
+def run_safe(_python_lambda, *args, **kwargs):
+    print_error = kwargs.get('print_error', False)
     try:
-        return _python_lambda(**kwargs)
+        return _python_lambda(*args, **kwargs)
     except Exception as e:
         if print_error:
             LOG.warning('Unable to execute function: %s' % e)
@@ -1197,9 +1208,8 @@ class SafeStringIO(io.StringIO):
 def clean_cache(file_pattern=CACHE_FILE_PATTERN,
         last_clean_time=last_cache_clean_time, max_age=CACHE_MAX_AGE):
 
-    mutex_clean.acquire()
-    time_now = now()
-    try:
+    with MUTEX_CLEAN:
+        time_now = now()
         if last_clean_time['time'] > time_now - CACHE_CLEAN_TIMEOUT:
             return
         for cache_file in set(glob.glob(file_pattern)):
@@ -1207,8 +1217,6 @@ def clean_cache(file_pattern=CACHE_FILE_PATTERN,
             if time_now > mod_time + max_age:
                 rm_rf(cache_file)
         last_clean_time['time'] = time_now
-    finally:
-        mutex_clean.release()
     return time_now
 
 
